@@ -44,14 +44,11 @@ pub(crate) trait CryptInit: Sized {
     /// Return the number of bytes that this state will require.
     fn seed_len() -> usize;
     /// Construct this state from a seed of the appropriate length.
-    ///
-    /// TODO: maybe this should return a Result rather than just
-    /// asserting that the length is correct.
-    fn initialize(seed: &[u8]) -> Self;
+    fn initialize(seed: &[u8]) -> Result<Self>;
     /// Initialize this object from a key generator.
     fn construct<K: super::handshake::KeyGenerator>(keygen: K) -> Result<Self> {
         let seed = keygen.expand(Self::seed_len())?;
-        Ok(Self::initialize(&seed))
+        Self::initialize(&seed)
     }
 }
 
@@ -195,7 +192,7 @@ impl InboundClientCrypt {
     /// Decrypt an incoming cell that is coming to the client.
     ///
     /// On success, return which hop was the originator of the cell.
-    // XXXX use real tag type
+    // TODO(nickm): Use a real type for the tag, not just `&[u8]`.
     pub(crate) fn decrypt(&mut self, cell: &mut RelayCellBody) -> Result<(HopNum, &[u8])> {
         for (hopnum, layer) in self.layers.iter_mut().enumerate() {
             if let Some(tag) = layer.decrypt_inbound(cell) {
@@ -259,8 +256,10 @@ pub(crate) mod tor1 {
         fn seed_len() -> usize {
             SC::KeySize::to_usize() * 2 + D::OutputSize::to_usize() * 2
         }
-        fn initialize(seed: &[u8]) -> Self {
-            assert!(seed.len() == Self::seed_len());
+        fn initialize(seed: &[u8]) -> Result<Self> {
+            if seed.len() != Self::seed_len() {
+                return Err(Error::InternalError("length is invalid".to_string()));
+            }
             let keylen = SC::KeySize::to_usize();
             let dlen = D::OutputSize::to_usize();
             let fdinit = &seed[0..dlen];
@@ -269,15 +268,15 @@ pub(crate) mod tor1 {
             let bckey = &seed[dlen * 2 + keylen..dlen * 2 + keylen * 2];
             let fwd = CryptState {
                 cipher: SC::new(fckey.try_into().expect("Wrong length"), &Default::default()),
-                digest: D::new().chain(fdinit),
+                digest: D::new().chain_update(fdinit),
                 last_digest_val: GenericArray::default(),
             };
             let back = CryptState {
                 cipher: SC::new(bckey.try_into().expect("Wrong length"), &Default::default()),
-                digest: D::new().chain(bdinit),
+                digest: D::new().chain_update(bdinit),
                 last_digest_val: GenericArray::default(),
             };
-            CryptStatePair { fwd, back }
+            Ok(CryptStatePair { fwd, back })
         }
     }
 
@@ -343,7 +342,8 @@ pub(crate) mod tor1 {
             self.0[8] = 0;
 
             d.update(&self.0[..]);
-            *used_digest = d.clone().finalize(); // XXX can I avoid this clone?
+            // TODO(nickm) can we avoid this clone?  Probably not.
+            *used_digest = d.clone().finalize();
             self.0[5..9].copy_from_slice(&used_digest[0..4]);
         }
         /// Check a cell to see whether its recognized field is set.
@@ -504,11 +504,11 @@ mod test {
 
         let mut cc_out = OutboundClientCrypt::new();
         let mut cc_in = InboundClientCrypt::new();
-        let pair = Tor1RelayCrypto::initialize(&K1[..]);
+        let pair = Tor1RelayCrypto::initialize(&K1[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
-        let pair = Tor1RelayCrypto::initialize(&K2[..]);
+        let pair = Tor1RelayCrypto::initialize(&K2[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
-        let pair = Tor1RelayCrypto::initialize(&K3[..]);
+        let pair = Tor1RelayCrypto::initialize(&K3[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
 
         let mut xof = tor_llcrypto::d::Shake256::default();
